--------------------------------------------------------------------------------------------
-- Broker_TSMProfile - Displays the current TSM profile and lets you switch between profiles
--------------------------------------------------------------------------------------------
-- Broker_TSMProfile.lua
--
--[[

Did You ever post a lot of items to the auction house using a wrong TSM profile?

Broker_TSMProfile displays the current TSM profile and lets you change to a 
different profile with a simple click, so you never post items with the wrong profile again.

This addon requires Trade Skill Master and has LDB support (Bazooka etc.).

Author: Caraxe
All Rights Reserved

Version 0.1.3
------------------------------------------------------------------------------------]]

-----------------------------------------------------------------------------
-- Initializing Addon

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local addonVersionString = "V" .. GetAddOnMetadata(addonName, "Version")
local LibQTip = LibStub("LibQTip-1.0")

-----------------------------------------------------------------------------
-- Encounter TSM object reference hiding

local AceAddon = LibStub("AceAddon-3.0")
local AceConfigReg = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local TSMAddonName = ""
local TSMAddonObject = nil

function NewAddonH(objectorname, ...)
	local o = NewAddonO(objectorname, ...)
	if o then
		if o.name == "TradeSkillMaster" then
			TSMAddonName = o.name
			TSMAddonObject = o
			-- found it, back to normal
			AceAddon.NewAddon = NewAddonO
		end
	end
	return o
end

if AceAddon then
	NewAddonO = AceAddon.NewAddon
	AceAddon.NewAddon = NewAddonH
end

-----------------------------------------------------------------------------
-- LDB Integration

local function do_OnEnter(frame)
	local tooltip = LibQTip:Acquire(addon)
	tooltip:SmartAnchorTo(frame)
	tooltip:SetAutoHideDelay(0.1, frame)
	tooltip:EnableMouse(true)
	addon:UpdateTooltip()
	tooltip:Show()
end

local function do_OnLeave()
	-- empty dummy
end

local function do_OnClick(frame, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	end
	addon:UpdateTooltip()
end

addon.LDB = LibStub("LibDataBroker-1.1"):NewDataObject(addonName, {
	type = "data source",
	label = addonName,
	text = L["TSM Profile"] .. ": --",
	icon = "Interface\\Icons\\Trade_Engineering",
	OnEnter = do_OnEnter,
	OnLeave = do_OnLeave,
	OnClick = do_OnClick,
});

local Icon = LibStub("LibDBIcon-1.0")

-----------------------------------------------------------------------------
-- Options

local defaults = {
	profile = {
		minimap = {
			hide = false,
		},
	},
	global = {},
}

local function GetOptions(uiType, uiName, appName)
	if appName == addonName then
		local options = {
				type = "group",
				name = GetAddOnMetadata(addonName, "Title"),
				args = {
					brfudesc = {
						type = "description",
						order = 0,
						name = GetAddOnMetadata(addonName, "Notes"),
					},
					minimap = {
						name = L["Minimap Icon"],
						desc = L["Toggle minimap icon"],
						type = "toggle",
						order = 10,
						get = function() return not db.minimap.hide end,
						set = function()
							db.minimap.hide = not db.minimap.hide
							if db.minimap.hide then
								Icon:Hide(addonName)
							else
								Icon:Show(addonName)
							end
						end,
					},
				},
		}
		return options
	end
end

-----------------------------------------------------------------------------
-- Broker_TSMProfile

local function SecTimer()
	local profile = ""
	if TSMAddonObject then
		profile = TSMAddonObject.db:GetCurrentProfile()
	end
	addon.LDB.text = L["TSM Profile"] .. ": " .. profile
end

function addon:OnInitialize()
	addon.db = LibStub("AceDB-3.0"):New("Broker_TSMProfileDB", defaults, true)
	db = self.db.profile
	Icon:Register(addonName, addon.LDB, db.minimap)
	
	addon.ExternalsInfo = ""
	local TSMV = GetAddOnMetadata("TradeSkillMaster", "Version")
	if (TSMV) then
		addon.ExternalsInfo = addon.ExternalsInfo .. "Trade Skill Master " .. TSMV
	end
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(addonName, GetOptions)
	local optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
end

function addon:OnEnable()
	if TSMAddonObject then
		-- for detecting externals switches
		addon.sectimer = C_Timer.NewTicker(1, SecTimer)

		local profile = TSMAddonObject.db:GetCurrentProfile()
		addon:Printf(addonVersionString .. ", " .. addon.ExternalsInfo .. L[" detected"] .. ", " .. L["TSM Profile"] .. ": " .. profile)

		-- leave it hidden
		-- AceAddon.addons[TSMAddonName] = TSMAddonObject
	else
		addon:Printf(addonVersionString .. ", " .. L["no TSM object found"] .. ".")
	end
end

local function tooltip_line_OnMouseUp(frame, profile, button)
	if TSMAddonObject then
		TSMAddonObject:ChangeProfile(profile)
		addon.LDB.text = L["TSM Profile"] .. ": " .. profile
	end
	addon:UpdateTooltip()
end

function addon:UpdateTooltip()
	if not LibQTip:IsAcquired(addon) then
		return
	end
	local tooltip = LibQTip:Acquire(addon)
	tooltip:Clear()
	tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

	local line = tooltip:AddHeader()
	tooltip:SetCell(line, 1, "|cfffed100Broker_|cffffffffTSMProfile", nil, "CENTER", 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, L["Profile"])
	tooltip:SetCell(line, 2, L["Status"])
	line = tooltip:AddSeparator()

	if TSMAddonObject then
		for i, profile in ipairs(TSMAddonObject.db:GetProfiles()) do
			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, format("%d. %s", i, profile))
			if (TSMAddonObject.db:GetCurrentProfile() == profile) then
				tooltip:SetCell(line, 2, L["On"])
			else
				tooltip:SetCell(line, 2, "--")
			end
			tooltip:SetLineScript(line, "OnMouseUp", tooltip_line_OnMouseUp, profile)
		end
	end

	line = tooltip:AddSeparator()
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Click to switch to profile"], nil, nil, 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Right-Click opens options"], nil, nil, 2)

	tooltip:UpdateScrolling()
end


-- EOF
